#!/usr/bin/perl -w
use strict;
use Data::Dumper;

  use GPG;
  use Treaty;

  my $VERBOSE = 1;
  my $DEBUG   = 0;

  my @test = qw/ test_creation_of_treaty test_creation_of_treaty_signed/;

  test();

######################################################

  sub debug { my ($msg) = @_;
    print "\n\n$msg\n------------------------------------------\n" if $VERBOSE;
  }

############################################################
#
# generic test suite (copied from pf, is this some default?)
#
############################################################
  sub test {
    my $count = -1;
    local $| = 1;
    for my $i (@test) {
      no strict 'refs';
      $count++;
      print "test ",sprintf("%2d",$count)," $i",substr("....................",0,(20-length($i)));
      eval { &$i };
      if($@) { 
        chomp($@);
        print " NOT ok -- $@\n";
      }
      else {
        print " ok.\n";
      }
    }
  }

  sub test_creation_of_treaty {
    my $creditor_id = "Linus linus\@penguniworld";
#    my $creditor_id = "Linus";
    my $debitor_id  = "Bill  bill\@megalomane";
#    my $debitor_id  = "Bill";
    my $amount      = "1000";
    my $currency    = "goodwill_credits";

    # create new treaty - basic
    # init with no contents
    my $treaty = new Treaty();
    die "1 - $treaty->error()" if $treaty->error();
    
    # fill in the treaty
    $treaty->contents($creditor_id,$debitor_id,$amount,$currency);
    die "2 - $treaty->error()" if $treaty->error();

    # test :
    # veryfy that the contents are what they should
    verify_contents($treaty, $creditor_id, $debitor_id, $amount, $currency);
  }

  sub test_creation_of_treaty_signed {
    my $cred_name  = "Linus";
    my $cred_email = "linus\@penguniworld";
    my $debi_name  = "Bill";
    my $debi_email = "bill\@megalomane";
    my $amount     = "1000";
    my $currency   = "goodwill_credits";
    my $passphrase = "testing glue";

    my $cred_id    = "$cred_name <$cred_email>";
    my $debi_id    = "$debi_name <$debi_email>";


    # create new treaty - extended
    # init with contents
    my $treaty = new Treaty($cred_id, $debi_id, $amount, $currency);
    die "3 - $treaty->error()" if $treaty->error();
    
    # test :
    # veryfy that the contents are what they should
    verify_contents($treaty, $cred_id, $debi_id, $amount, $currency);

    # create new GPG environment
    my $gpg = new GPG(homedir  => './test');
    die "4 - $gpg->error()" if $gpg->error();
    
    # create new creditor key
    my ($cred_pubring,$cred_secring) = $gpg->gen_key(#key_size   => "256",
                                                     key_size   => "512",
                                                     real_name  => $cred_name,
                                                     email      => $cred_email,
                                                     comment    => "",
                                                     passphrase => $passphrase);
    die "5 - $gpg->error()" if $gpg->error();

    my $cred_pubkeys = $gpg->list_packets($cred_pubring);
    die "6 - $gpg->error()" if $gpg->error();
    my $cred_key_id = $cred_pubkeys->[0]{'key_id'};
    die "7 - $gpg->error()" if $gpg->error();
    

    # create new debitor key
    my ($debi_pubring,$debi_secring) = $gpg->gen_key(#key_size   => "256",
                                                     key_size   => "512",
                                                     real_name  => $debi_name,
                                                     email      => $debi_email,
                                                     comment    => "",
                                                     passphrase => $passphrase);
    die "8 - $gpg->error()" if $gpg->error();

    my $debi_pubkeys = $gpg->list_packets($debi_pubring);
    die "9 - $gpg->error()" if $gpg->error();
    my $debi_key_id = $debi_pubkeys->[0]{'key_id'};
    die "10 - $gpg->error()" if $gpg->error();
    
    
    # export both keys to gpg
    $gpg->write_keys($cred_secring);
    die "11 - $gpg->error()" if $gpg->error();
    $gpg->write_keys($cred_pubring);
    die "12 - $gpg->error()" if $gpg->error();
    #
    $gpg->write_keys($debi_secring);
    die "13 - $gpg->error()" if $gpg->error();
    $gpg->write_keys($debi_pubring);
    die "14 - $gpg->error()" if $gpg->error();
	     
    $treaty->sign($gpg, $cred_key_id,$passphrase);
    die "15 - $treaty->error()" if $treaty->error();
    $treaty->sign($gpg, $debi_key_id,$passphrase);
    die "16 - $treaty->error()" if $treaty->error();
    $treaty->sign($gpg, $debi_key_id,$passphrase);
    die "17 - There should've been an error here,".
            " because we've signed a Treaty twice" unless $treaty->error();
  }

####################################################################
#
# check that the contents are what they should
#
####################################################################
  sub verify_contents {
    my ($treaty, $creditor_id,$debitor_id,$amount,$currency) = @_;

    debug "The promise is as follows: ".$treaty->promise()."\n";
    die "13 - $treaty->error()" if $treaty->error();

    my ($creditor_id_check,$debitor_id_check,$amount_check,$currency_check) =
      $treaty->contents();
    die "14 - $treaty->error()" if $treaty->error();

    if (     $creditor_id eq $creditor_id_check
         and $debitor_id  eq $debitor_id_check
         and $amount      eq $amount_check
         and $currency     eq $currency_check) {
      debug "After set and get the values are still the same, hurray! :-)\n";}
    else {
      die "After set and get the values are not the same any more :-(!!\n".
          ": $creditor_id_check,$debitor_id_check,$amount_check,$currency_check"; }

  }

# End of 'test.pl'.
